val overflow_bound_max_int32_value : int32
val int32_positive_overflow_bounds : int32 array
val overflow_bound_max_int_value : int
val int_positive_overflow_bounds : int array
val overflow_bound_max_int63_on_int64_value : int64
val int63_on_int64_positive_overflow_bounds : int64 array
val overflow_bound_max_int64_value : int64
val int64_positive_overflow_bounds : int64 array
val int64_negative_overflow_bounds : int64 array
