open! Import0

type t = Stdlib.Uchar.t

let succ = Stdlib.Uchar.succ
let pred = Stdlib.Uchar.pred
let is_valid = Stdlib.Uchar.is_valid
let is_char = Stdlib.Uchar.is_char
let unsafe_to_char = Stdlib.Uchar.unsafe_to_char
let unsafe_of_int = Stdlib.Uchar.unsafe_of_int
let of_int = Stdlib.Uchar.of_int
let to_int = Stdlib.Uchar.to_int
let of_char = Stdlib.Uchar.of_char
let compare = Stdlib.Uchar.compare
let equal = Stdlib.Uchar.equal
let min_value = Stdlib.Uchar.min
let max_value = Stdlib.Uchar.max
